@@ -5,10 +5,7 @@ use warnings;
 use lib 'lib';
 
 package File::Util;
-{
-  $File::Util::VERSION = '4.130610'; # TRIAL
-}
-
+$File::Util::VERSION = '4.160630'; # TRIAL
 use File::Util::Definitions qw( :all );
 use File::Util::Interface::Modern qw( :all );
 
@@ -26,6 +23,7 @@ our @EXPORT_OK  = qw(
    is_bin  created     last_access   last_changed  last_modified
    isbin   split_path  atomize_path  diagnostic    abort_depth
    size    can_read    can_write     read_limit    can_utf8
+                       default_path  strict_path
 );
 
 our %EXPORT_TAGS = ( all => [ @EXPORT_OK ], diag => [ ] );
@@ -1156,7 +1154,7 @@ sub load_file {
       $this->_seize( $clean_name, $fh, $in );
    }
 
-   # call binmode on binary files for portability accross platforms such
+   # call binmode on binary files for portability across platforms such
    # as MS flavor OS family
 
    binmode $fh if -B $clean_name;
@@ -1177,10 +1175,14 @@ sub load_file {
             return $this->_throw( 'no unicode' => $in );
          }
       }
-      else
+      elsif ( $in->{binmode} == 1 )
       {
          binmode $fh;
       }
+      else
+      {
+         binmode $fh, $in->{binmode} # apply user-specified IO layer(s)
+      }
    }
 
    # assign the content of the file to this lexically scoped scalar variable
@@ -1428,24 +1430,45 @@ sub write_file {
    if ( $in->{no_lock} || !$USE_FLOCK ) {
 
       # only non-existent files get bitmask arguments
-      if ( -e $clean_name ) {
-
-         sysopen
-            $write_fh,
-            $clean_name,
-            $$MODES{sysopen}{ $mode }
-         or return $this->_throw(
+      if ( -e $clean_name )
+      {
+         # you can't use UTF8 'mode' on system IO, so if a user requests
+         # UTF8, we have to use PerlIO
+         if ( $in->{binmode} && lc $in->{binmode} eq 'utf8' )
+         {
+            open
+               $write_fh,
+               $$MODES{popen}{ $mode },
+               $clean_name
+            or return $this->_throw(
                'bad open'   => {
                   filename  => $clean_name,
                   mode      => $mode,
-                  exception => $!,
-                  cmd       => qq($clean_name, $$MODES{sysopen}{ $mode }),
                   opts      => $in,
+                  exception => $!,
+                  cmd       => $mode . $clean_name,
                }
             );
+         }
+         else
+         {
+            sysopen
+               $write_fh,
+               $clean_name,
+               $$MODES{sysopen}{ $mode }
+            or return $this->_throw(
+               'bad open'   => {
+                  filename  => $clean_name,
+                  mode      => $mode,
+                  opts      => $in,
+                  exception => $!,
+                  cmd       => qq($clean_name, $$MODES{sysopen}{ $mode }),
+               }
+            );
+         }
       }
-      else {
-
+      else
+      {
          sysopen
             $write_fh,
             $clean_name,
@@ -1462,56 +1485,100 @@ sub write_file {
          );
       }
    }
-   else {
+   else
+   {
       # open read-only first to safely check if we can get a lock.
-      if ( -e $clean_name ) {
-
-         open $write_fh, '<', $clean_name or
-            return $this->_throw(
-               'bad open'   => {
-                  filename  => $clean_name,
-                  mode      => 'read',
-                  exception => $!,
-                  cmd       => $mode . $clean_name,
-                  opts      => $in,
-               }
-            );
+      if ( -e $clean_name )
+      {
+         open $write_fh, '<', $clean_name
+         or return $this->_throw(
+            'bad open'   => {
+               filename  => $clean_name,
+               mode      => 'read',
+               exception => $!,
+               cmd       => $mode . $clean_name,
+               opts      => $in,
+            }
+         );
 
          # lock file before I/O on platforms that support it
          my $lockstat = $this->_seize( $clean_name, $write_fh, $in );
 
          return unless $lockstat;
 
-         sysopen
-            $write_fh,
-            $clean_name,
-            $$MODES{sysopen}{ $mode }
-         or return $this->_throw(
-            'bad open'   => {
-               filename  => $clean_name,
-               mode      => $mode,
-               opts      => $in,
-               exception => $!,
-               cmd       => qq($clean_name, $$MODES{sysopen}{ $mode }),
-            }
-         );
+         # you can't use UTF8 'mode' on system IO, so if a user requests
+         # UTF8, we have to use PerlIO
+         if ( $in->{binmode} && lc $in->{binmode} eq 'utf8' )
+         {
+            open
+               $write_fh,
+               $$MODES{popen}{ $mode },
+               $clean_name
+            or return $this->_throw(
+               'bad open'   => {
+                  filename  => $clean_name,
+                  mode      => $mode,
+                  opts      => $in,
+                  exception => $!,
+                  cmd       => $mode . $clean_name,
+               }
+            );
+         }
+         else
+         {
+            sysopen
+               $write_fh,
+               $clean_name,
+               $$MODES{sysopen}{ $mode }
+            or return $this->_throw(
+               'bad open'   => {
+                  filename  => $clean_name,
+                  mode      => $mode,
+                  opts      => $in,
+                  exception => $!,
+                  cmd       => qq($clean_name, $$MODES{sysopen}{ $mode }),
+               }
+            );
+         }
       }
       else { # only non-existent files get bitmask arguments
+             # ...unless doing utf8 business, in which case it's irrelevant
 
-         sysopen
-            $write_fh,
-            $clean_name,
-            $$MODES{sysopen}{ $mode },
-            $bitmask
-         or return $this->_throw(
-            'bad open'   => {
-               filename  => $clean_name,
-               mode      => $mode,
-               opts      => $in,
-               exception => $!,
-               cmd       => qq($clean_name, $$MODES{sysopen}{$mode}, $bitmask),
-            }
-         );
+         # you can't use UTF8 'mode' on system IO, so if a user requests
+         # UTF8, we have to use PerlIO
+         if ( $in->{binmode} && lc $in->{binmode} eq 'utf8' )
+         {
+            open
+               $write_fh,
+               $$MODES{popen}{ $mode },
+               $clean_name
+            or return $this->_throw(
+               'bad open'   => {
+                  filename  => $clean_name,
+                  mode      => $mode,
+                  opts      => $in,
+                  exception => $!,
+                  cmd       => $mode . $clean_name,
+               }
+            );
+         }
+         else
+         {
+            sysopen
+               $write_fh,
+               $clean_name,
+               $$MODES{sysopen}{ $mode },
+               $bitmask
+            or return $this->_throw(
+               'bad open'   => {
+                  filename  => $clean_name,
+                  mode      => $mode,
+                  opts      => $in,
+                  exception => $!,
+                  cmd       => qq($clean_name, $$MODES{sysopen}{$mode}, $bitmask),
+               }
+            );
+         }
 
          # lock file before I/O on platforms that support it
          my $lockstat = $this->_seize( $clean_name, $write_fh, $in );
@@ -1539,6 +1606,8 @@ sub write_file {
          if ( $HAVE_UU )
          {
             binmode $write_fh, ':unix:encoding(UTF-8)';
+
+            print $write_fh $content; # utf8 filehandles use PerlIO
          }
          else
          {
@@ -1547,13 +1616,23 @@ sub write_file {
             return $this->_throw( 'no unicode' => $in );
          }
       }
-      else
+      elsif ( $in->{binmode} == 1 )
       {
          binmode $write_fh;
+
+         syswrite( $write_fh, $content );
+      }
+      else
+      {
+         binmode $write_fh, $in->{binmode}; # apply user-specified IO layer(s)
+
+         syswrite( $write_fh, $content );
       }
    }
-
-   syswrite( $write_fh, $content );
+   else
+   {
+      syswrite( $write_fh, $content );
+   }
 
    # release lock on the file
 
@@ -1664,7 +1743,7 @@ sub atomize_path {
 
 
 # --------------------------------------------------------
-# File::Util::atomize_path()
+# File::Util::split_path()
 # --------------------------------------------------------
 sub split_path {
    my $path = _myargs( @_ );
@@ -1809,7 +1888,7 @@ sub touch {
       'cant touch on a dir',
       {
          filename => $file,
-         dirname  => $path || '',
+         dirname  => $path,
          opts     => $opts,
       }
    ) if -e $file && -d $file;
@@ -2513,6 +2592,27 @@ sub open_handle {
          }
       );
    }
+
+   # Final bit of input validation made necessary by the would-be perils
+   # of IO encoding while using sys(open,read,write,seek,tell,etc) -
+   # Basically, using :utf8 encoding with syswrite is deprecated
+   if
+   (
+      ( exists $in->{use_sysopen} && defined $in->{use_sysopen} ) &&
+      ( $in->{binmode} && lc $in->{binmode} eq 'utf8' )
+   )
+   {
+      return $this->_throw(
+         'bad binmode',
+         {
+            meth    => 'open_handle',
+            filename => $clean_name,
+            dirname  => $root . $path,
+            opts     => $in,
+         }
+      );
+   }
+
    # input validation sequence finished
 
    if ( $$in{no_lock} || !$USE_FLOCK ) {
@@ -2539,14 +2639,14 @@ sub open_handle {
          # get open mode
          $mode = $$MODES{sysopen}{ $mode };
 
-         sysopen( $fh, $clean_name, $$MODES{sysopen}{ $mode } ) or
+         sysopen( $fh, $clean_name, $mode ) or
             return $this->_throw(
                'bad open',
                {
                   filename  => $clean_name,
                   mode      => $mode,
                   exception => $!,
-                  cmd       => qq($clean_name, $$MODES{sysopen}{ $mode }),
+                  cmd       => qq($clean_name, $mode),
                   opts      => $in,
                }
             );
@@ -2685,10 +2785,14 @@ sub open_handle {
             return $this->_throw( 'no unicode' => $in );
          }
       }
-      else
+      elsif ( $in->{binmode} == 1 )
       {
          binmode $fh;
       }
+      else
+      {
+         binmode $fh, $in->{binmode} # apply user-specified IO layer(s)
+      }
    }
 
    # return file handle reference to the caller
@@ -2720,7 +2824,42 @@ sub unlock_open_handle {
 # --------------------------------------------------------
 # File::Util::return_path()
 # --------------------------------------------------------
-sub return_path { my $f = _myargs( @_ ); $f =~ s/(^.*)$DIRSPLIT.*/$1/o; $f }
+sub return_path { my $f = _myargs( @_ ); $f =~ s/(^.*)$DIRSPLIT.*/$1/; $f }
+
+
+# --------------------------------------------------------
+# File::Util::strict_path()
+# --------------------------------------------------------
+sub strict_path
+{
+   my $path = _myargs( @_ );
+   my $copy = $path;
+
+   ( $path ) = $path =~ /(^.*$DIRSPLIT)/;
+
+   ( $path ) = $copy =~ /(^\.{1,2}$)/ if !defined $path;
+
+   return unless defined $path;
+
+   $path .= SL unless substr $path, -1, 1 =~ /$DIRSPLIT/;
+
+   return $path =~ /$DIRSPLIT/ ? $path : undef;
+}
+
+
+# --------------------------------------------------------
+# File::Util::default_path()
+# --------------------------------------------------------
+sub default_path
+{
+   my ( $path, $dflt ) = _myargs( @_ );
+
+   $dflt = defined $dflt ? $dflt : '.' . SL;
+
+   $path = strict_path( $path );
+
+   return defined $path ? $path : $dflt;
+}
 
 
 # --------------------------------------------------------
@@ -2853,7 +2992,7 @@ File::Util - Easy, versatile, portable file handling
 
 =head1 VERSION
 
-version 4.130610
+version 4.160630
 
 =head1 DESCRIPTION
 
@@ -2864,7 +3003,10 @@ have to worry about whether their programs will work on other operating systems
 and/or architectures.  It works on Linux, Windows, Mac, BSD, Unix and others.
 
 File::Util is written B<purely in Perl>, and requires no compiler or make
-utility on your system in order to install and run it.
+utility on your system in order to install and run it.  It loads a minimal
+amount of code when used, only pulling in support for lesser-used methods
+on demand.  It has no dependencies other than what comes installed with Perl
+itself.
 
 File::Util also aims to be as backward compatible as possible, running without
 problems on Perl installations as old as 5.006.  You are encouraged to run
@@ -2898,20 +3040,20 @@ reference materials:
 
 =over
 
-=item B<The "Nutshell">
+=item B<Examples>
 
 The L<File::Util::Manual::Examples> document has a long list of small, reusable
 code snippets and techniques to use in your own programs.  This is the "cheat
 sheet", and is a great place to get started quickly.  Almost everything you
 need is here.
 
-=item B<The Manual>
+=item B<Complete API Reference>
 
 The L<File::Util::Manual> is the complete reference document explaining every
 available feature and object method.  Use this to look up the full information
 on any given feature when the examples aren't enough.
 
-=item B<The Cookbook>
+=item B<Cookbook>
 
 The L<File::Util::Cookbook> contains examples of complete, working programs
 that use File::Util to easily accomplish tasks which require file handling.
@@ -2937,22 +3079,16 @@ that use File::Util to easily accomplish tasks which require file handling.
 
 =head2 File Operations
 
-   # load content into a variable, be it text, or binary, either works
-   my $content = $f->load_file( 'data.txt' );
+   # load file content into a scalar variable as raw text
+   my $content = $f->load_file( 'somefile.txt' );
 
-   # wrangle some text
-   $content =~ s/this/that/g;
+   # read a binary file the same way
+   my $binary_content = $f->load_file( 'barking-cat.mp4' );
 
-   # write a file with your changes
-   $f->write_file( 'new_data.txt' => $content );
+   # write a raw text file
+   $f->write_file( 'somefile.txt' => $content );
 
-   # try binary this time
-   my $binary_content = $f->load_file( 'barking-cat.avi' );
-
-   # get some image data from somewhere...
-   my $picture_data = get_image_upload();
-
-   # ...and write a binary image file, using some other options as well
+   # ...and write a binary file, using some other options as well
    $f->write_file(
       'llama.jpg' => $picture_data => { binmode => 1, bitmask => oct 644 }
    );
@@ -3006,9 +3142,19 @@ that use File::Util to easily accomplish tasks which require file handling.
 
    syswrite $fh, "that's no moon";
 
-   # ...you can use any of these syswrite modes, also with { binmode => 'utf8' }
+   # ...You can use any of these syswrite modes with open_handle():
    # read, write, append, rwcreate, rwclobber, rwappend, rwupdate, and trunc
 
+PLEASE NOTE that as of Perl 5.23, it is deprecated to mix system IO
+(sysopen/syswrite/sysread/sysseek) with utf8 binmode (see perldoc perlport).
+As such, File::Util will no longer allow you to do this after version
+4.132140.  Please see notes on UTF-8 and encoding further below.
+
+   # ...THIS WILL NOW FAIL!
+   $f->open_handle(
+      'somefile.txt' => 'write' => { use_sysopen => 1, binmode => 'utf8' }
+   );
+
 =head2 Directories
 
    # get a listing of files, recursively, skipping directories
@@ -3067,16 +3213,83 @@ that use File::Util to easily accomplish tasks which require file handling.
 pattern matching in directories, callbacks, directory walking, user-definable
 error handlers, and more.
 
+=head2 File Encoding and UTF-8
+
+If you want to read/write in UTF-8, you can do that:
+
+   $ftl->load_file( 'file.txt' => $content => { binmode => 'utf8' } );
+
+   $ftl->write_file( 'file.txt' => $content => { binmode => 'utf8' } );
+
+   $ftl->open_handle( 'file.txt' => 'read' => { binmode => 'utf8' } );
+
+   # ...and so on
+
+Only use C<binmode =E<gt> 'utf8'> for text.
+
+Encoding and IO layers (sometimes called disciplines) can become complex.
+It's not something you usually need to worry about unless you wish to
+really fine tune File::Util's behavior beyond what are very suitable, portable
+defaults, or accomplish very specific tasks like encoding conversions.
+
+You're free to specify any binmode you like, or allow File::Util to use the
+system's default IO layering.  It will automatically use the ":raw" pseudo layer
+when reading files that are binary, unless specifically told to use something
+different.
+
+You can control things as shown in the examples below:
+
+   $ftl->load_file( 'file.txt' => $content => { binmode => SPEC } );
+
+   $ftl->write_file( 'file.txt' => $content => { binmode => SPEC } );
+
+   $ftl->open_handle( 'file.txt' => 'read' => { binmode => SPEC } );
+
+...where C<SPEC> is one or more of any supported IO layers on your system.
+Examples might include:
+
+=over
+
+=item *
+
+C<':raw'>
+
+=item *
+
+C<':unix'>
+
+=item *
+
+C<':crlf'>
+
+=item *
+
+C<':stdio'>
+
+=item *
+
+C<':encoding(ENCODING)'> I<with ENCODING's like iso-8859-1, shiftjis, etc>
+
+=item *
+
+...and much more
+
+=back
+
+You can learn about the IO layers available to you and what they do in the
+L<PerlIO> perldoc.  Available options have increased over the years, and are
+likely subject to continued evolution.  Consult the L<PerlIO> and L<Encode>
+documentation as your authoritative source of info on what layers to use.
+
 =head1 PERFORMANCE
 
-File::Util consists of several modules, but only loads the ones it needs when
-it needs them and also offers a comparatively fast load-up time, so using
-File::Util doesn't bloat your code footprint.
+File::Util consists of a set of smaller modules, but only loads the ones it
+needs when it needs them.  It offers a comparatively fast load-up time, so using
+File::Util doesn't bloat your code's resource footprint.
 
-Additionally, File::Util has been optimized to run fast.*  In many scenarios
-it does more and still out-performs other popular IO modules from anywhere
-from 100%-400%, although L<Path::Tiny> is also extremely fast at what it is
-designed to do.
+Additionally, File::Util has been optimized to run fast.  In many scenarios
+it does more and still out-performs other popular IO modules.  Benchmarking tools
+are included as part of the File::Util installation package.
 
 I<(See the benchmarking and profiling scripts>
 I<that are included as part of this distribution.)>
@@ -3103,6 +3316,8 @@ this document in a text terminal, open perldoc to the C<File::Util::Manual>.
 
 =item created              I<(see L<created|File::Util::Manual/created>)>
 
+=item default_path         I<(see L<default_path|File::Util::Manual/default_path>)>
+
 =item diagnostic           I<(see L<diagnostic|File::Util::Manual/diagnostic>)>
 
 =item ebcdic               I<(see L<ebcdic|File::Util::Manual/ebcdic>)>
@@ -3155,6 +3370,8 @@ this document in a text terminal, open perldoc to the C<File::Util::Manual>.
 
 =item split_path           I<(see L<split_path|File::Util::Manual/split_path>)>
 
+=item strict_path          I<(see L<strict_path|File::Util::Manual/strict_path>)>
+
 =item strip_path           I<(see L<strip_path|File::Util::Manual/strip_path>)>
 
 =item touch                I<(see L<touch|File::Util::Manual/touch>)>
@@ -3178,18 +3395,18 @@ You can, however, ask it for certain things (below).
 
 =head2 EXPORT_OK
 
-The following symbols comprise C<@File::Util::EXPORT_OK>), and as such are
+The following symbols comprise C<@File::Util::EXPORT_OK>, and as such are
 available for import to your namespace only upon request.  They can be
 used either as object methods or like regular subroutines in your program.
 
-   -  atomize_path        -  can_flock         -  can_utf8
-   -  created             -  diagnostic        -  ebcdic
-   -  escape_filename     -  existent          -  file_type
-   -  is_bin              -  is_readable       -  is_writable
-   -  last_access         -  last_changed      -  last_modified
-   -  needs_binmode       -  return_path       -  size
-   -  split_path          -  strip_path        -  valid_filename
-   -  NL and S L
+   -  atomize_path      -  can_flock         -  can_utf8
+   -  created           -  default_path      -  diagnostic
+   -  ebcdic            -  escape_filename   -  existent
+   -  file_type         -  is_bin            -  is_readable
+   -  is_writable       -  last_access       -  last_changed
+   -  last_modified     -  needs_binmode     -  strict_path
+   -  return_path       -  size              -  split_path
+   -  strip_path        -  valid_filename    -  NL and S L
 
 To get any of these functions/symbols into your namespace without having
 to use them as object methods, use this kind of syntax:
@@ -3228,6 +3445,10 @@ You can technically run File::Util on older versions of Perl 5, but it isn't
 recommended, especially if you want unicode support and wish to take advantage
 of File::Util's ability to read and write files using UTF-8 encoding.
 
+L<Unicode::UTF8> is also recommended and helps speed things up
+in several places where you might choose to use unicode as described
+elsewhere in the L<File::Util::Manual>.
+
 =back
 
 =head1 INSTALLATION
