@@ -4,10 +4,7 @@ use warnings;
 use lib 'lib';
 
 package File::Util::Exception::Standard;
-{
-  $File::Util::Exception::Standard::VERSION = '4.130610'; # TRIAL
-}
-
+$File::Util::Exception::Standard::VERSION = '4.160630'; # TRIAL
 # ABSTRACT: Standard (non-verbose) error messages
 
 use File::Util::Definitions qw( :all );
@@ -227,6 +224,13 @@ Call to $EBL$opts->{meth}()$EBR failed: @{[
 __no_input__
 
 
+# CAN'T USE UTF8 WITH SYSOPEN
+'bad binmode' => <<'__bad_binmode__',
+The use of system IO (sysread/syswrite/etc) on utf8 file handles is deprecated.
+Please don't specify { use_sysopen => 1 } together with { binmode => 'utf8' }
+__bad_binmode__
+
+
 # PLAIN ERROR TYPE
 'plain error' => <<'__plain_error__',
 ${\ scalar ($_[0] || ((exists $opts->{error} && defined $opts->{error}) ?
@@ -273,7 +277,7 @@ File::Util::Exception::Standard - Standard (non-verbose) error messages
 
 =head1 VERSION
 
-version 4.130610
+version 4.160630
 
 =head1 DESCRIPTION
 
